'use strict';
const apimap = require('./apimap');
const GitUtil = require('./git');
const cwd = process.cwd();
const fs = require('fs-extra');
const gitInfo = GitUtil.getProjectAndGroupName();
const collector = require('nuke-collector');
const request = require('request');
const path = require('path');
const chalk = require('chalk');
const inquirer = require('inquirer');
const fieApi = require('fie-api');
const fieConfig = fieApi.config;

let authorInfo = require('fie-user').getUser();

/**
 * 新增场景
 */
exports.new = function(publish_type, env) {
  const ENV = env || 'development';
  const API = Object.assign({}, apimap.base, apimap[ENV]);
  const API_THEME_NEW = API._HOST + API['theme.new'];

  let pkgInfo = require(path.resolve(cwd, 'package.json'));
  let collectResult = collector.collect({
    rootDir: cwd,
    basicPackage: pkgInfo.themeOption.baseNuke,
    businessComponentsRegExp:
      pkgInfo.themeOption.baseNuke == 'nuke' ? null : /@ali\/nuke\-/
  });
  let packages = [];
  let keywords =
    !publish_type || publish_type === 'theme'
      ? ['nuke-theme']
      : ['nuke-template'];
  let components = collectResult.statistics.basicComponentsTimes;
  Object.keys(components).forEach(key => {
    let obj = {};
    obj[key] = components[key];
    packages.push(obj);
  });
  if (!pkgInfo.description || pkgInfo.description.length < 10) {
    console.log(
      chalk.blue(
        'ABORT:多写点儿description吧，这会帮助使用者更了解你的(场景/项目)功能哦'
      )
    );
    // console.log(chalk.yellow('修改package.json的对应字段后可通过fie update-theme更新(场景/项目)信息'));
  }

  if (!pkgInfo.keywords) {
    console.log(
      chalk.blue(
        'ABORT:麻烦增加一些keywords把，这可以帮助我们更好的归类你的(场景/项目)'
      )
    );
    // console.log(chalk.yellow('修改package.json的对应字段后可通过fie update-theme更新场景信息'));
  }

  let args = {
    name: pkgInfo.name,
    git: `${gitInfo.groupName}/${gitInfo.projectName}`,
    source: 'gitlab',
    description: pkgInfo.description,
    type: publish_type || 'theme',
    baseNuke: pkgInfo.themeOption.baseNuke,
    screenshot: JSON.stringify(pkgInfo.themeOption.screenshot),
    pageName: pkgInfo.themeOption.pageName,
    packages: JSON.stringify(packages),
    author: authorInfo.name,
    authorEmail: authorInfo.email,
    authorId: '',
    packageInfo: JSON.stringify(pkgInfo),
    isShow: 1,
    keywords: pkgInfo.keywords
      ? JSON.stringify(keywords.concat(pkgInfo.keywords))
      : JSON.stringify(keywords),
    compatibility: pkgInfo.compatibility
      ? JSON.stringify(pkgInfo.compatibility)
      : '{}',
    version: pkgInfo.version || '0.0.1'
  };

  request.post(API_THEME_NEW, { form: args, json: true }, function(
    error,
    response,
    body
  ) {
    if (error) return console.error(error);
    if (body.success) {
      pkgInfo.themeOption.id = body.data.id;
      console.log(
        chalk.green(
          'SUCCESS:接口发布成功，现在开发者可以在场景市场中查看到你发布的场景啦~'
        )
      );
      fs.writeJson(
        path.resolve(cwd, 'package.json'),
        pkgInfo,
        { spaces: 4 },
        function(err) {
          if (err) return console.log(err);
          console.log(chalk.green('SUCCESS:场景信息回写成功'));
        }
      );
    } else {
      if (body.code == 103) {
        console.log('INFO:接口入参错误，请确定输入参数');
        console.log(chalk.red('ERROR:接口请求失败，请联系@督布'));
      } else if (body.code == '10120' || body.code == '10121') {
        inquirer
          .prompt([
            {
              name: 'option',
              type: 'confirm',
              default: true,
              message: chalk.blue.bold(
                'INFO:已存在相同name或git地址的场景,是否进行更新操作?'
              )
            }
          ])
          .then(option => {
            if (option.option) {
              args.id = pkgInfo.themeOption.id;
              exports.update(args, ENV);
            }
          });
      } else {
        console.log(chalk.red('INFO:未知错误，请联系@督布'));
      }
    }
  });
};
/**
 * 更新场景
 */
exports.update = function(obj, env) {
  const ENV = env || 'development';
  const API = Object.assign({}, apimap.base, apimap[ENV]);
  const API_THEME_UPDATE = API._HOST + API['theme.update'];
  request.post(API_THEME_UPDATE, { form: obj, json: true }, function(
    error,
    response,
    body
  ) {
    if (error) return console.error(error);
    if (body.success) {
      console.log(chalk.green('SUCCESS:更新成功'));
    } else {
      console.log(chalk.red('ERROR:更新失败'));
      console.log(`${body.code}:${body.msg}`);
    }
  });
};
