/*
 *  为package.json添加兼容性描述
 */
'use strict';
const inquirer = require('inquirer');
const path = require('path');
const fs = require('fs-extra');
const chalk = require('chalk');
const semver = require('semver');
const cwd = process.cwd();
module.exports = function*() {
  var pkgObj = require(path.resolve(process.cwd(), 'package.json'));
  var PARAMS = [
    {
      name: 'qn-ios',
      text:
        '[千牛-ios]兼容性,兼容所有版本<true>,不兼容<false>,只兼容此版本以上<x.y.z>',
      default: () => {
        if (!pkgObj.compatibility) {
          return 'true';
        } else {
          return pkgObj.compatibility.qn && pkgObj.compatibility.qn.ios
            ? pkgObj.compatibility.qn.ios
            : 'true';
        }
      }
    },
    {
      name: 'qn-android',
      text:
        '[千牛-android]兼容性,兼容所有版本<true>,不兼容<false>,只兼容此版本以上<x.y.z>',
      default: () => {
        if (!pkgObj.compatibility) {
          return 'true';
        } else {
          return pkgObj.compatibility.qn && pkgObj.compatibility.qn.android
            ? pkgObj.compatibility.qn.android
            : 'true';
        }
      }
    },
    {
      name: 'tb-ios',
      text:
        '[淘宝-ios]兼容性,兼容所有版本<true>,不兼容<false>,只兼容此版本以上<x.y.z>',
      default: () => {
        if (!pkgObj.compatibility) {
          return 'true';
        } else {
          return pkgObj.compatibility.tb && pkgObj.compatibility.tb.ios
            ? pkgObj.compatibility.tb.ios
            : 'true';
        }
      }
    },
    {
      name: 'tb-android',
      text:
        '[淘宝-android]兼容性,兼容所有版本<true>,不兼容<false>,只兼容此版本以上<x.y.z>',
      default: () => {
        if (!pkgObj.compatibility) {
          return 'true';
        } else {
          return pkgObj.compatibility.tb && pkgObj.compatibility.tb.android
            ? pkgObj.compatibility.tb.android
            : 'true';
        }
      }
    },
    {
      name: 'tm-ios',
      text:
        '[天猫-ios]兼容性,兼容所有版本<true>,不兼容<false>,只兼容此版本以上<x.y.z>',
      default: () => {
        if (!pkgObj.compatibility) {
          return 'true';
        } else {
          return pkgObj.compatibility.tm && pkgObj.compatibility.tm.ios
            ? pkgObj.compatibility.tm.ios
            : 'true';
        }
      }
    },
    {
      name: 'tm-android',
      text:
        '[天猫-android]兼容性,兼容所有版本<true>,不兼容<false>,只兼容此版本以上<x.y.z>',
      default: () => {
        if (!pkgObj.compatibility) {
          return 'true';
        } else {
          return pkgObj.compatibility.tm && pkgObj.compatibility.tm.android
            ? pkgObj.compatibility.tm.android
            : 'true';
        }
      }
    }
  ];
  const questions = PARAMS.map(param => {
    const question = {
      type: 'input',
      message: chalk.blue(param.text),
      name: param.name,
      validate: input => {
        return input !== '';
      }
    };
    // default
    if (param.default) {
      question.default = () => {
        return param.default();
      };
    }
    return question;
  });
  if (!pkgObj.compatibility) {
    console.log(chalk.green('未进行过兼容性设置，开始设置:'));
  } else {
    return console.log(
      chalk.green(
        '已存在兼容性设置，如需修改请手动修改package.json中的compatibility字段'
      )
    );
  }
  let result = yield inquirer.prompt(questions);
  Object.keys(result).forEach(item => {
    if (result[item] != 'true' && result[item] != 'false') {
      if (!semver.valid(result[item])) {
        console.log(
          chalk.red(`${result[item]}不是语义化的版本规范，请按照x.y.z规则命名`)
        );
        process.exit(1);
      }
    }
  });
  pkgObj.compatibility = {
    qn: {
      ios: result['qn-ios'],
      android: result['qn-android']
    },
    tb: {
      ios: result['tb-ios'],
      android: result['tb-android']
    },
    tm: {
      ios: result['tm-ios'],
      android: result['tm-android']
    }
  };
  let fsOk = fs.writeJsonSync(path.join(cwd, 'package.json'), pkgObj, {
    spaces: 4
  });
  if (fsOk) return console.log(chalk.red('写package.json失败'));
  console.log(chalk.green('兼容性设置成功'));
};
