/**
 * Created by hugo on 16/5/10.
 */
'use strict';
var fs = require('fs');
var path = require('path');
var gitlab = require('node-gitlab');
var spawn = require('cross-spawn');
var semver = require('semver');
var _ = require('underscore');
var cwd = process.cwd();
var request = require('co-request');
var api = require('fie-api');
let fieHost = 'http://fie-api.alibaba-inc.com';

var config = {
  api: 'http://gitlab.alibaba-inc.com/api/v3',
  //fie_admin的token
  privateToken: 'E8JD26cJrdQ93MiZGQFv'
};
var client = gitlab.createPromise(config);
/**
 * 搜索gitlab上面的project
 * @param option 搜索条件
 *     group 搜索该group下面的仓库
 *     name 搜索的仓库名
 *     page 搜索到第几页
 * @param callback 回调
 */
function searchProject(option, callback) {
  if (!option.page) option.page = 1;
  client.projects
    .search({ query: option.name, per_page: 100, page: option.page })
    .then(function(projects) {
      var len = projects.length;

      var project;
      for (var i = 0; i < len; i++) {
        var item = projects[i];
        if (item.namespace.name === option.group && item.name == option.name) {
          project = item;
          break;
        }
      }
      //只递归10页,避免无限递归
      if (!project && option.page <= 10 && len) {
        option.page++;
        searchProject(option, callback);
      } else {
        callback && callback(project);
      }
    })
    .catch(err => {
      console.log(err);
    });
}

module.exports = {
  fieGetProject: function*() {
    const auth = require('@ali/fie-auth');
    const token = yield auth.token();
    const project = this.getProjectId();
    const response = yield request({
      uri: `${fieHost}/gitlab/projects/${project}/members`,
      json: true,
      qs: {
        token
      }
    });
    if (response && response.statusCode === 200 && response.body) {
      return response.body;
    }
    return null;
  },
  getProjectId: function() {
    const that = this;
    const gitInfo = that.getProjectAndGroupName();
    return `${gitInfo.groupName}%2F${gitInfo.projectName}`;
  },
  /**
   * 同步package.json的版本号为当前分支的版本
   */
  syncPackageVersion: function() {
    var file = path.join(cwd, 'package.json'),
      pkg,
      branch;

    if (fs.existsSync(file)) {
      pkg = require(file);
      branch = this.getCurBranch();
      branch = branch.replace('daily/', '');
      //判断一下 是否是标准的 x.y.z格式
      if (semver.valid(branch)) {
        pkg.version = branch;
        fs.writeFileSync(file, JSON.stringify(pkg, null, 2), {
          encoding: 'utf8'
        });
        return branch;
      }
    }
  },

  /**
   * 打开当前项目URL
   */
  openProject: function() {
    var file = path.join(cwd, '.git/config'),
      url;

    if (fs.existsSync(file)) {
      var gitConfig = fs.readFileSync(file, { encoding: 'utf8' });
      var match = gitConfig.match(/git@gitlab.alibaba-inc.com:(.*)\.git/);
      if (match && match[1]) {
        url = 'http://gitlab.alibaba-inc.com/' + match[1];
      }
    }
    return url;
  },

  /**
   * 获取当前分支
   * @returns {string}
   */
  getCurBranch: function() {
    if (process.env.BUILD_ENV === 'cloud') {
      return process.env.BUILD_GIT_BRANCH;
    }
    var headerFile = path.join(cwd, '.git/HEAD');
    var gitVersion =
      (fs.existsSync(headerFile) &&
        fs.readFileSync(headerFile, { encoding: 'utf8' })) ||
      '';
    var arr = gitVersion.split(/refs[\\\/]heads[\\\/]/g);
    var v = (arr && arr[1]) || '';
    return v.trim();
  },
  /**
   * 判断本地是否有当前daily分支
   * @method function
   * @return {[type]} [description]
   */

  isBranchExist: function(version) {
    var headerFile = path.join(cwd, `.git/refs/heads/daily/${version}`);
    console.log(headerFile);
    var gitVersion = fs.existsSync(headerFile); // && fs.readFileSync(headerFile, {encoding: 'utf8'}) || '';
    return gitVersion;
  },

  /**
   * 获取package中的desction字段
   * @returns {string|*}
   */
  getDescription: function() {
    var file = path.join(cwd, 'package.json'),
      pkg,
      description = '';

    if (fs.existsSync(file)) {
      pkg = require(file);
      description = pkg.description;
    }
    return description;
  },

  /**
   * 获取project name 和 group name
   * @returns {*}
   */
  getProjectAndGroupName: function() {
    if (process.env.BUILD_ENV === 'cloud') {
      return {
        groupName: process.env.BUILD_GIT_GROUP,
        projectName: process.env.BUILD_GIT_PROJECT
      };
    }
    var file = path.join(cwd, '.git/config');
    var pkgFile = path.join(cwd, 'package.json');
    var pg;
    var gitConfig;
    var match;

    if (fs.existsSync(file)) {
      gitConfig = fs.readFileSync(file, { encoding: 'utf8' });
      if (!/git@gitlab.alibaba-inc.com:(.*)/.test(gitConfig)) {
        gitConfig = '';
      }
    }

    if (!gitConfig && fs.existsSync(pkgFile)) {
      gitConfig = require(pkgFile).repository;
      if (gitConfig && typeof gitConfig !== 'string') {
        gitConfig = gitConfig.url;
      }
    }

    if (gitConfig) {
      match = gitConfig.match(/git@gitlab.alibaba-inc.com:(.*)/);
      if (match && match[1]) {
        pg = match[1].split('/');
        return {
          groupName: pg[0],
          projectName: pg[1].replace('.git', '')
        };
      }
    }
    return {};
  },

  /**
   * 获取project Id
   * @returns {number}
   */
  getProject: function*() {
    var that = this;
    var gitInfo = that.getProjectAndGroupName();
    return new Promise(function(resolve, reject) {
      searchProject(
        {
          group: gitInfo.groupName,
          name: gitInfo.projectName
        },
        function(project) {
          console.log('$$$$', project);
          resolve(project);
        }
      );
    });
  },

  /**
   * 获取用户信息
   */
  getUserInfo: function() {
    var results = spawn.sync('git', ['config', '--list']);
    var data = {
      author: '',
      email: ''
    };
    try {
      //分成2个正则
      var regUser = /user\.name=([^\n]+)\n/;
      var regEmail = /user\.email=([^\n]+)\n/;
      var matchUser = results.stdout.toString().match(regUser);
      var matchEmail = results.stdout.toString().match(regEmail);
      // console.log(match,'==match')
      if (matchUser) {
        data.author = matchUser[1];
      }
      if (matchEmail) {
        data.email = matchEmail[1];
      }
    } catch (e) {}

    return data;
  },

  /**
   * 根据namepace 获取 id
   */
  getNamespaceIdByName: function(name) {
    //namespaces
    return new Promise(function(resolve, reject) {
      request(
        {
          url:
            config.api +
            '/namespaces?per_page=100&private_token=' +
            config.privateToken, //http://gitlab.alibaba-inc.com/api/v3/namespaces?private_token=4k3Yx3xVx3xkhUENyQtC',
          timeout: 5000,
          json: true
        },
        function(error, response, body) {
          var id;
          if (body && body.length) {
            body.forEach(function(item) {
              if (item.path === name) {
                id = item.id;
              }
            });

            resolve(id);
          } else {
            reject(Error('error'));
          }
        }
      );
    });
  },

  /**
   * 根据当前项目中的git信息,在gitlab上创建项目
   */
  createProject: function() {
    var that = this;
    var gitInfo = that.getProjectAndGroupName();
    var gitConfigDir = path.join(cwd, '.git');

    return new Promise(function(resolve, reject) {
      Promise.all([
        that.getProject(),
        that.getNamespaceIdByName(gitInfo.groupName)
      ]).then(
        function(data) {
          //不存在仓库,才创建
          if (data && data[0]) {
            resolve(_.extend(gitInfo, { error: 'project' }));
          } else if (data && !data[1]) {
            resolve(_.extend(gitInfo, { error: 'ns' }));
          } else {
            var description = that.getDescription();

            if (!fs.existsSync(gitConfigDir)) {
              spawn.sync('git', ['init']);
              spawn.sync('git', [
                'config',
                'remote.origin.url',
                'git@gitlab.alibaba-inc.com:' +
                  gitInfo.groupName +
                  '/' +
                  gitInfo.projectName +
                  '.git'
              ]);
              spawn.sync('git', [
                'remote.origin.fetch',
                '+refs/heads/*:refs/remotes/origin/*'
              ]);
              spawn.sync('git', ['branch.master.remote', 'origin']);
              spawn.sync('git', ['branch.master.merge', 'refs/heads/master']);
            }

            client.projects.create(
              {
                name: gitInfo.projectName,
                description: description,
                namespace_id: data[1],
                public: true
              },
              function(response) {
                if (
                  !response ||
                  response.statusCode == 200 ||
                  response.statusCode == undefined
                ) {
                  resolve(gitInfo);
                } else {
                  resolve(_.extend(gitInfo, { error: 'create' }));
                }
              }
            );
          }
        },
        function(data) {
          resolve(_.extend(gitInfo, { error: 'project' }));
        }
      );
    });
    //查一下 看是否有这个仓库
  },

  /**
   * 获取该项目下所有 有权限的用户
   */
  listUsers: function() {
    var that = this;
    var gitInfo = that.getProjectAndGroupName();
    return new Promise(function(resolve, reject) {
      Promise.all([
        that.getProject(),
        that.getNamespaceIdByName(gitInfo.groupName)
      ]).then(function(data) {
        client.projectMembers
          .list({
            id: data[0].id
          })
          .then(
            function(pUser) {
              return pUser;
            },
            function() {
              return [];
            }
          )
          .then(function(pUser) {
            client.groupMembers
              .list({
                id: data[1]
              })
              .then(
                function(gUser) {
                  var users = gUser.concat(pUser);
                  resolve(users);
                },
                function() {
                  resolve(pUser);
                }
              );
          });
      });
    });
  },

  /**
   * 获取user信息
   */
  getUser: function() {
    var that = this;
    return new Promise(function(resolve, reject) {
      var userInfo = that.getUserInfo();
      if (userInfo && userInfo.email) {
        //client
        client.users
          .list({
            search: userInfo.email
          })
          .then(function(data) {
            if (data && data.length) {
              data[0].email = userInfo.email;
              resolve(data[0]);
            } else {
              //gitlab上没有找到该用户
              reject(new Error('no'));
            }
          });
      } else {
        //当前机器上没有安装git
        reject(new Error('git'));
      }
    });
  },

  addUserToProject: function() {
    var that = this;
    // console.log(123)
    return new Promise(function(resolve, reject) {
      //获取user 和 project 信息 再添加用户到project
      Promise.all([that.getUser(), that.getProject()]).then(function(data) {
        if (data && data[0] && data[1]) {
          client.projectMembers
            .create({
              user_id: data[0].id,
              id: data[1].id,
              access_level: 40
            })
            .then(function(response) {
              if (response && response.id) {
                resolve(data);
              } else {
                reject(Error('create'));
              }
            });
        } else {
          reject(Error('create'));
        }
      });
    });
  },

  /**
   * 判断线上文件是否存在
   * @param url
   */
  checkFileExit: function(url) {},

  publishCDN: function(cwd, version) {
    //git 发布流
    var gitCommands = [
      'git,add,-A',
      'git,commit,-m,temp: update',
      'git,push,origin,daily/' + version,
      'git,tag,publish/' + version,
      'git,push,origin,publish/' + version + ':publish/' + version,
      'git,checkout,master',
      'git,merge,publish/' + version,
      'git,push'
    ];
    let succssFlag = true;
    gitCommands.forEach(function(item, i) {
      const command = item.split(',');

      const results = spawn.sync(command.splice(0, 1).pop(), command, {
        cwd: cwd,
        env: process.env,
        stdio: 'inherit'
      });

      if (results.status == 0 || results.status == 1) {
        api
          .log('Publish Info')
          .success(item.replace(/,/g, ' ') + ' 命令执行成功');
        return true;
      } else {
        succssFlag = false;
        api.log('Publish Info').error('发布失败');
        process.exit(1);
      }
    });
    return succssFlag;
  }
};
