'use strict';
const request = require('request');
const koa = require('koa');
const path = require('path');
const gitclone = require('gitlab-clone');
const ejs = require('ejs');
const fs = require('fs-extra');
const Util = require('./util');
const chalk = require('chalk');
const co = require('co');
const cors = require('koa-cors');
const open = require('open');
const fieApi = require('fie-api');
const fieEnv = require('fie-env');
const userHome =
  process.env.HOME || process.env.USERPROFILE || process.env.HOMEPATH; //兼容windows
const qs = require('qs');
const BayBridge = require('bay-bridge');
const isIntranet = fieEnv.isIntranet();
const COMMON_NUKE = 'weex-nuke';

module.exports.getAliTheme = function(obj) {
  return BayBridge.getMtopData({
    api: 'mtop.taobao.nuke.theme.list.get',
    v: '1.0',
    data: obj
  });
};

module.exports.startThemeServer = function*(params) {
  let current = this;
  let themeSource = yield this.getAliTheme({
    openStatus: isIntranet ? -1 : 1
  });
  const templatePath = path.resolve(__dirname, '../tpl/theme-market.html');

  var app = koa();
  app.use(cors());
  app.use(function*() {
    let that = this;
    let url = this.request.url;
    let query = this.request.query;
    if (url.indexOf('theme/install') > -1) {
      let targetTheme = Util.getTarget(themeSource, query.name);
      let srcBranch = targetTheme.version
        ? `#publish/${targetTheme.version}`
        : '';
      params.git = targetTheme.git;
      that.body = yield current.autoInstall(targetTheme);
    } else if (url.indexOf('theme/mergeApimap') > -1) {
      //交互式合并apimap
      console.log('开始合并啦');
      let targetTheme = Util.getTarget(themeSource, query.name);
      let localPath = path.join(
        path.resolve(userHome, `.fie_toolkit_temp/${targetTheme.pageName}`)
      );
      if (query.merge) {
        that.body = Util.mergeApimap(localPath, true);
      }
    } else if (url.indexOf('theme/skip') > -1) {
      console.log('关闭场景市场中...');
      that.body = {
        success: true
      };
      process.exit(0);
    } else {
      ejs.renderFile(templatePath, {}, function(err, str) {
        if (err) {
          console.log(err);
          return (that.body = err);
        } else {
          return (that.body = str);
        }
      });
    }
  });
  app.listen(4545);
  setTimeout(function() {
    params.baseNuke ? params.baseNuk : (params.baseNuk = 'nuke');
    open(
      `http://h5.m.taobao.com/qn/nuke-market.html?baseNuke=${
        params.baseNuke
      }&&isIntranet=${isIntranet}`
    );

    // open(`http://localhost:4545/theme.html?baseNuke=${params.baseNuke}`);
    console.log(chalk.green('场景市场服务已启动~'));
  }, 1000);
};

module.exports.installAliTheme = function(
  localPath,
  destPath,
  themeInfo,
  autoMerge
) {
  let targePath = path.join(localPath, 'src/pages/index');
  let current = this;
  return new Promise((resolve, reject) => {
    fs.copy(targePath, destPath)
      .then(function() {
        console.log(chalk.green(`已完成src目录的拷贝,开始合并依赖...`));
        let srcPkgPath = path.join(localPath, 'package.json');
        let destPkgPath = path.resolve(process.cwd(), 'package.json');

        try {
          let srcPkgInfo = require(srcPkgPath);
          let destPkgInfo = require(destPkgPath);

          for (var key in srcPkgInfo['dependencies']) {
            if (!destPkgInfo['dependencies'][key]) {
              // @ali/nuke 场景合并到weex-nuke
              if (key == 'nuke' || key == COMMON_NUKE || key == '@ali/nuke') {
                let baseNuke =
                  key == COMMON_NUKE || key === '@ali/nuke'
                    ? 'nuke'
                    : COMMON_NUKE;
                Util.replaceBaseNuke(baseNuke, themeInfo);
              } else {
                destPkgInfo['dependencies'][key] =
                  srcPkgInfo['dependencies'][key];
                console.log(
                  chalk.yellow(`发现本地不存在的依赖---${key},开始写入依赖树`)
                );
              }
            }
          }

          fs.outputJson(destPkgPath, destPkgInfo, { spaces: 2 }, function(err) {
            if (err) {
              console.log(`依赖写入失败，请手动添加`);
              reject(err);
            }
            console.warn(`依赖写入成功,重新安装中...`);
            co(function*() {
              yield fieApi.npm.installDependencies();
            }).then(() => {
              console.log(
                chalk.yellow(
                  `安装完成，control + c 停止服务，重新运行fie start`
                )
              );
            });
          });
        } catch (error) {
          console.log(`未找到目标依赖，依赖合并终止，请手动添加`);
          console.log(error);
          reject(error);
        }
        resolve(Util.mergeApimap(localPath, autoMerge));
      })
      .catch(err => {
        console.log(chalk.red('文件拷贝失败，请手动安装'));
        console.log(err);
        reject(error);
      });
  });
};

module.exports.clientInstall = function(localPath, params) {
  var Core0 = require('@ali/0-client-core');
  var core = new Core0();
  return new Promise(function(resolve, reject) {
    core
      .useAliGitlabSeed({
        gitPath: params.git,
        outputPath: localPath,
        params: params
      })
      .then(function() {
        console.log(chalk.green('拉取远程场景成功'));
        resolve({ success: true });
      })
      .catch(function(error) {
        console.log(chalk.red('拉取远程场景失败，请确认目标仓库public可读'));
        reject({ success: false, msg: error });
      });
  });
};

module.exports.clone = function*(localPath, destPath, target, merge) {
  let current = this;
  let srcBranch = target.version ? `#publish/${target.version}` : '';
  return yield new Promise((resolve, reject) => {
    let gitAddress = '';
    if (!isIntranet) {
      gitAddress = target.git.replace('nuke-theme', 'weex-nuke');
    } else {
      gitAddress = `${target.git}${srcBranch}`;
    }
    gitclone(
      gitAddress,
      { dest: localPath, ssh: false, ali: isIntranet },
      function(err, info) {
        if (err) {
          err.buffer
            ? console.log(err.buffer.toString())
            : console.log('远程仓库拉取失败,请通过查看源码，手动添加场景');
          reject({ success: false, msg: err });
        } else if (target.type === 'theme') {
          current
            .installAliTheme(localPath, destPath, target, merge)
            .then(res => {
              resolve(res);
            })
            .catch(error => {
              reject({ success: false, msg: error });
            });
        } else {
          /**
           * @todo 递归读写复制文件，避免非同名文件被删除,目前暂时使用安装前确认提示。
           */
          fs.emptyDirSync(path.resolve(process.cwd(), 'src'));
          fs.copy(
            localPath,
            process.cwd(),
            {
              filter: (src, dest) => {
                if (src.indexOf('.git') > -1) {
                  return false;
                }
                return true;
              }
            },
            err => {
              if (err) reject({ success: false, msg: err });
              resolve({ success: true });
            }
          );
        }
      }
    );
  });
};

module.exports.search = function*(obj) {
  let res = yield this.getAliTheme(obj);
  return res;
};

module.exports.autoInstall = function*(targetTheme, merge) {
  let res = {};
  let current = this;
  let srcBranch = targetTheme.version ? `#publish/${targetTheme.version}` : '';
  if (targetTheme.type === 'theme') {
    // 场景
    let localPath = path.join(
      path.resolve(userHome, `.fie_toolkit_temp/${targetTheme.pageName}`)
    );
    let destPath = path.resolve(
      process.cwd(),
      `src/pages/${targetTheme.pageName}`
    );
    fs.emptyDirSync(localPath);
    res = yield current.clone(localPath, destPath, targetTheme, merge);
  } else {
    // 项目模板
    let res = {};
    let localPath = path.join(
      path.resolve(userHome, `.fie_toolkit_temp/${targetTheme.name}`)
    );
    fs.emptyDirSync(localPath);
    if (targetTheme.source === 'gitlab') {
      res = yield current.clone(localPath, '', targetTheme, merge);
    } else if (targetTheme.source === '0-client') {
      res = yield current.clientInstall(process.cwd(), params);
    }
    yield fieApi.npm.installDependencies();
    console.log(
      chalk.yellow(`安装完成，control + c 停止服务，重新运行fie start`)
    );
  }
  return res;
};
