'use strict';
const babel = require('babel-core');
const fs = require('fs-extra');
const chalk = require('chalk');
const deepEql = require('deep-eql');
const path = require('path');
const walk = require('walk-fs');
const objectMerge = require('object-merge');
/**
 * 将目标模块代码es5化后获得Object
 */
module.exports.es6Toes5 = function(fileContent) {
  return eval(
    babel.transform(fileContent, {
      comments: true,
      ast: false,
      presets: [require.resolve('babel-preset-es2015')]
    }).code
  );
};
/**
 * 根据本地项目信息替换场景中的nuke依赖
 */
module.exports.replaceBaseNuke = function(baseNuke, themeInfo) {
  //遍历js和jsx文件，分析themeInfo中的nuke引用，替换为baseNuke.
  let srcBaseNuke = themeInfo.baseNuke == 'both' ? 'nuke' : themeInfo.baseNuke;
  let reg = new RegExp('import {.*} from ("|\')' + srcBaseNuke + '("|\')', 'g');
  let themePath = path.join(
    process.cwd(),
    'src/pages/',
    themeInfo.pageName + '/'
  );
  walk(
    themePath,
    { recursive: true },
    function(absoultePath, stats) {
      if (stats.isFile()) {
        if (path.extname(absoultePath).match(/\.(js|jsx)/)) {
          let content = fs.readFileSync(absoultePath, 'utf8');
          content = content.replace(reg, function(words) {
            return words.replace(srcBaseNuke, baseNuke);
          });
          fs.writeFileSync(absoultePath, content, 'utf8');
        }
      }
    },
    function(err) {
      if (err) {
        console.log(chalk.red(`${srcBaseNuke}自动替换失败，请手动修改引用`));
        return console.log(err);
      } else {
        console.log(chalk.green(`${srcBaseNuke}自动替换${baseNuke}完成~`));
      }
    }
  );
};
/**
 * 合并场景与本地项目的apimap
 */
module.exports.mergeApimap = function(localPath, autoMerge) {
  try {
    let srcApimapPath = path.resolve(localPath, 'src/util/apimap.js');
    let destApimapPath = path.resolve(process.cwd(), 'src/util/apimap.js');
    let srcApimap = '';
    let destApimap = '';
    try {
      srcApimap = fs.readFileSync(srcApimapPath, 'utf8');
    } catch (err) {
      // console.log('log:读文件失败',err);
      srcApimap = 'export default {}';
    }
    try {
      destApimap = fs.readFileSync(destApimapPath, 'utf8');
    } catch (err) {
      // console.log('log:读文件失败',err);
      destApimap = 'export default {}';
    }
    let srcModule = this.es6Toes5(srcApimap);
    let destModule = this.es6Toes5(destApimap);

    if (deepEql(srcModule, destModule)) {
      console.log(chalk.green('apimap一致，无需合并'));
      return { success: true };
    } else if (
      (typeof srcModule !== 'object' || this.isObjEmpty(srcModule)) &&
      !autoMerge
    ) {
      return { success: true, info: '远程场景无全局apimap.js' };
    } else {
      console.log(
        chalk.red('当前选择场景有本地不存在的全局apimap配置,请手动添加')
      );
      if (!autoMerge) {
        return {
          success: true,
          continue: true,
          info: '当前选择场景有本地不存在的全局apimap配置,请手动添加'
        };
      } else {
        return {
          success: true,
          continue: true,
          info: '当前选择场景有本地不存在的全局apimap配置,请手动添加'
        };
        // let newApiModule = objectMerge(srcModule,destModule);
        // newApiModule = `export default ${JSON.stringify(newApiModule, null, 2)}`;
        // fs.writeFileSync(destApimapPath, newApiModule, 'utf8');
        // console.log(chalk.green('apimap合并完成'));
        // return {success:true,info:'mergeSuccess'};
      }
    }
  } catch (error) {
    console.log(
      chalk.red(`本地或远程场景apimap.js缺失，apimap合并跳过`),
      error
    );
    return { success: true, msg: error };
  }
};

module.exports.getTarget = function(source, name) {
  source = source.filter(item => {
    return item.name === name;
  });
  return source[0];
};

module.exports.isObjEmpty = function(obj) {
  for (var name in obj) {
    if (obj.hasOwnProperty(name)) {
      return false;
    }
  }
  return true;
};
